function [fbest sol]=trustregion(objFcnHandle,xinit,varargin)

%Default parameters
mindelta = 1e-04;
delta = 1;
maxiter = 100;
etag = 0.9;
etaa = 0.1;
gammae = 2;
gammas = 0.5;
gtol=1e-05;
mtol=1e-05;
hessdelta=1e-03;

%Initialization
x=xinit;
[current,g,H] = objFcnHandle(x,varargin{:});
best=current;
solution = x;
gradlength = norm(g); 

%Main loop
iter = 0;
while iter < maxiter && gradlength > gtol
%     disp(['Iteration =' num2str(iter) ' Obj. function value = ' num2str(best) ' Current point = ' num2str(x) ]);

    %Indefinite Hessian treatment
    [R,pd]=chol(H);
    if pd==1 
      H=H+(abs(min(eig(H)))+hessdelta)*eye(length(x));
    end
    
    %Step computation
    %Newton's step
    N = (-inv(H)*g')';
    if norm(N) <= delta
      p = N;
      %disp(['N'])
    else 
      %Exact steepest descent step
      S = -(gradlength^2/(g*(H*g'))).*g;
      Slength = norm(S);
      if Slength > delta
	   p = delta.*(S/Slength);
	   %disp(['C'])
      else
        %Dogleg intersection 
	    [tau, ftau, info] = fzero (@(tau) norm(S+tau*(N-S))-delta,[0 1]);
	    if info == 1
	      p = S+tau*(N-S);
	      %disp(['Dogleg with tau = ' num2str(tau)])
        else
	      p = S; %If search for tau fails, use steepest descent step
	      %disp(['S'])
	    end
      end
    end    
    
   %Test prediction quality
   [candidate,cg,cH] =  objFcnHandle(x+p,varargin{:});
   if (current-m(current,g,H,p)) < mtol
    %The current iterate may be in an almost flat area (result of poor scaling)
    rho=0;
   else
    rho=(current-candidate)/(current-m(current,g,H,p));
   end

   %Accept or reject step and adapt trust region radius
   if rho >= etag
    x = x+p;
    delta = delta*gammae;
    
    best=candidate;
    solution = x;
    g=cg;
    gradlength = norm(g);
    H=cH;
   else
    if rho >= etaa
      x=x+p;
      
      best=candidate;
      solution = x;
      gradlength = norm(g);
      g=cg;
      H=cH;
    else 
	delta = gammas*delta;
	if delta < mindelta
	  delta = mindelta;
	end
    end
   end  
  iter = iter + 1;
end

sol=solution;
fbest=best;

end